package com.logicbig.example;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

public class EmployeeUpdateWithoutForceIncrement {
    private static EntityManagerFactory entityManagerFactory =
            Persistence.createEntityManagerFactory("example-unit");

    public static void main(String[] args) throws Exception {
        ExecutorService es = Executors.newFixedThreadPool(2);
        try {
            persistEmployee();
            es.execute(() -> {
                employeeUpdate();
            });
            es.execute(() -> {
                //simulating other user update
                departmentUpdate();
            });
            es.shutdown();
            es.awaitTermination(10, TimeUnit.SECONDS);
        } finally {
            entityManagerFactory.close();
        }
    }

    private static void employeeUpdate() {
        EntityManager em = entityManagerFactory.createEntityManager();
        Employee employee = em.find(Employee.class, 1L);
        em.getTransaction().begin();
        employee.setSalary(3000);
        try {
            TimeUnit.SECONDS.sleep(1);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        System.out.println("-- updating employee --");
        em.getTransaction().commit();
        em.close();
        System.out.println("Employee updated 1: " + employee);
    }

    private static void departmentUpdate() {
        EntityManager em = entityManagerFactory.createEntityManager();
        Department department = em.find(Department.class, 2);
        em.getTransaction().begin();
        department.setName("Admin");
        System.out.println("-- updating department --");
        em.getTransaction().commit();
        em.close();
        System.out.println("Department updated: " + department);
    }

    public static void persistEmployee() {
        Employee employee = new Employee("Joe", 2000, Department.create("Account"));
        EntityManager em = entityManagerFactory.createEntityManager();
        em.getTransaction().begin();
        em.persist(employee);
        System.out.println("-- persisting employee --");
        em.getTransaction().commit();
        em.close();
        System.out.println("Employee persisted: " + employee);
    }
}